#include "HasMQTT.hpp"

// C++ system level
// #include <functional>

// ESP32 specific
// #include "esp_log.h"

// project specific
#include <Types.hpp>

// qthing stuff
#include <qthing>
#include <qthing/mqtt_common.hpp>

// misc
// #include ""


static const str slash = "/";


namespace SpiderLib {


  bool HasMQTT::mqttConnected() const {
    return qthing::is_mqtt_connected();
  }

  bool HasMQTT::mqttPublish(const str topic, const str message, bool retained) {
    bool connected = qthing::is_mqtt_connected();
    if (connected) qthing::publish_message(topic, message, retained);
    return connected;
  }

  void HasMQTT::mqttSubscribe(const str& topic, HasMQTT::StringMsgHandler callback) {
    qthing::add_message_callback(topic, callback);
  }





  str HasMQTT::deviceTopic() const {
    // TODO: adapt this when qthing changes from macro to real string
    str topic = DEVICE_NAMESPACE;
    topic.pop_back();  // erase the last `/` from qthing

    return topic;
  }

  str HasMQTT::deviceTopic(const str& subt0) const {
    str topic = deviceTopic();

    topic += slash + subt0;

    return topic;
  }

  str HasMQTT::deviceTopic(const str& subt0, const str& subt1) const {
    str topic = deviceTopic();

    topic += slash + subt0;
    topic += slash + subt1;

    return topic;
  }

  str HasMQTT::deviceTopic(const str& subt0, const str& subt1, const str& subt2) const {
    str topic = deviceTopic();

    topic += slash + subt0;
    topic += slash + subt1;
    topic += slash + subt2;

    return topic;
  }

  str HasMQTT::deviceTopic(const str& subt0, const str& subt1, const str& subt2, const str& subt3) const {
    str topic = deviceTopic();

    topic += slash + subt0;
    topic += slash + subt1;
    topic += slash + subt2;
    topic += slash + subt3;

    return topic;
  }

  str HasMQTT::deviceTopic(const str& subt0, const str& subt1, const str& subt2, const str& subt3, const str& subt4) const {
    str topic = deviceTopic();

    topic += slash + subt0;
    topic += slash + subt1;
    topic += slash + subt2;
    topic += slash + subt3;
    topic += slash + subt4;

    return topic;
  }

  str HasMQTT::deviceTopic(const str& subt0, const str& subt1, const str& subt2, const str& subt3, const str& subt4, const str& subt5) const {
    str topic = deviceTopic();

    topic += slash + subt0;
    topic += slash + subt1;
    topic += slash + subt2;
    topic += slash + subt3;
    topic += slash + subt4;
    topic += slash + subt5;

    return topic;
  }

  str HasMQTT::deviceTopic(const str& subt0, const str& subt1, const str& subt2, const str& subt3, const str& subt4, const str& subt5, const str& subt6) const {
    str topic = deviceTopic();

    topic += slash + subt0;
    topic += slash + subt1;
    topic += slash + subt2;
    topic += slash + subt3;
    topic += slash + subt4;
    topic += slash + subt5;
    topic += slash + subt6;

    return topic;
  }

  str HasMQTT::deviceTopic(const str& subt0, const str& subt1, const str& subt2, const str& subt3, const str& subt4, const str& subt5, const str& subt6, const str& subt7) const {
    str topic = deviceTopic();

    topic += slash + subt0;
    topic += slash + subt1;
    topic += slash + subt2;
    topic += slash + subt3;
    topic += slash + subt4;
    topic += slash + subt5;
    topic += slash + subt6;
    topic += slash + subt7;

    return topic;
  }

  str HasMQTT::deviceTopic(const str& subt0, const str& subt1, const str& subt2, const str& subt3, const str& subt4, const str& subt5, const str& subt6, const str& subt7, const str& subt8) const {
    str topic = deviceTopic();

    topic += slash + subt0;
    topic += slash + subt1;
    topic += slash + subt2;
    topic += slash + subt3;
    topic += slash + subt4;
    topic += slash + subt5;
    topic += slash + subt6;
    topic += slash + subt7;
    topic += slash + subt8;

    return topic;
  }

  str HasMQTT::deviceTopic(const str& subt0, const str& subt1, const str& subt2, const str& subt3, const str& subt4, const str& subt5, const str& subt6, const str& subt7, const str& subt8, const str& subt9) const {
    str topic = deviceTopic();

    topic += slash + subt0;
    topic += slash + subt1;
    topic += slash + subt2;
    topic += slash + subt3;
    topic += slash + subt4;
    topic += slash + subt5;
    topic += slash + subt6;
    topic += slash + subt7;
    topic += slash + subt8;
    topic += slash + subt9;

    return topic;
  }



  str HasMQTT::serviceTopic() const {
    str topic = "service";
    return topic;
  }

  str HasMQTT::serviceTopic(const str& subt0) const {
    str topic = serviceTopic();

    topic += slash + subt0;

    return topic;
  }

  str HasMQTT::serviceTopic(const str& subt0, const str& subt1) const {
    str topic = serviceTopic();

    topic += slash + subt0;
    topic += slash + subt1;

    return topic;
  }

  str HasMQTT::serviceTopic(const str& subt0, const str& subt1, const str& subt2) const {
    str topic = serviceTopic();

    topic += slash + subt0;
    topic += slash + subt1;
    topic += slash + subt2;

    return topic;
  }

  str HasMQTT::serviceTopic(const str& subt0, const str& subt1, const str& subt2, const str& subt3) const {
    str topic = serviceTopic();

    topic += slash + subt0;
    topic += slash + subt1;
    topic += slash + subt2;
    topic += slash + subt3;

    return topic;
  }

  str HasMQTT::serviceTopic(const str& subt0, const str& subt1, const str& subt2, const str& subt3, const str& subt4) const {
    str topic = serviceTopic();

    topic += slash + subt0;
    topic += slash + subt1;
    topic += slash + subt2;
    topic += slash + subt3;
    topic += slash + subt4;

    return topic;
  }

  str HasMQTT::serviceTopic(const str& subt0, const str& subt1, const str& subt2, const str& subt3, const str& subt4, const str& subt5) const {
    str topic = serviceTopic();

    topic += slash + subt0;
    topic += slash + subt1;
    topic += slash + subt2;
    topic += slash + subt3;
    topic += slash + subt4;
    topic += slash + subt5;

    return topic;
  }

  str HasMQTT::serviceTopic(const str& subt0, const str& subt1, const str& subt2, const str& subt3, const str& subt4, const str& subt5, const str& subt6) const {
    str topic = serviceTopic();

    topic += slash + subt0;
    topic += slash + subt1;
    topic += slash + subt2;
    topic += slash + subt3;
    topic += slash + subt4;
    topic += slash + subt5;
    topic += slash + subt6;

    return topic;
  }

  str HasMQTT::serviceTopic(const str& subt0, const str& subt1, const str& subt2, const str& subt3, const str& subt4, const str& subt5, const str& subt6, const str& subt7) const {
    str topic = serviceTopic();

    topic += slash + subt0;
    topic += slash + subt1;
    topic += slash + subt2;
    topic += slash + subt3;
    topic += slash + subt4;
    topic += slash + subt5;
    topic += slash + subt6;
    topic += slash + subt7;

    return topic;
  }

  str HasMQTT::serviceTopic(const str& subt0, const str& subt1, const str& subt2, const str& subt3, const str& subt4, const str& subt5, const str& subt6, const str& subt7, const str& subt8) const {
    str topic = serviceTopic();

    topic += slash + subt0;
    topic += slash + subt1;
    topic += slash + subt2;
    topic += slash + subt3;
    topic += slash + subt4;
    topic += slash + subt5;
    topic += slash + subt6;
    topic += slash + subt7;
    topic += slash + subt8;

    return topic;
  }

  str HasMQTT::serviceTopic(const str& subt0, const str& subt1, const str& subt2, const str& subt3, const str& subt4, const str& subt5, const str& subt6, const str& subt7, const str& subt8, const str& subt9) const {
    str topic = serviceTopic();

    topic += slash + subt0;
    topic += slash + subt1;
    topic += slash + subt2;
    topic += slash + subt3;
    topic += slash + subt4;
    topic += slash + subt5;
    topic += slash + subt6;
    topic += slash + subt7;
    topic += slash + subt8;
    topic += slash + subt9;

    return topic;
  }

  // specialization end

}
